#  Copyright 2022 Google LLC
#  Copyright 2022 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import logging
import os
from typing import Tuple
from osdu_api.providers.blob_storage import get_client
from osdu_api.providers.types import FileLikeObject

from bundles.storage import BundleStorageClient

logger = logging.getLogger(__name__)


class GCBundleStorageClient(BundleStorageClient):
    def __init__(self) -> None:
        self.client = get_client()
        self.bucket_name = os.environ["POLICY_BUCKET"]
        self.content_type = "application/x-gtar"

    def _get_gs_uri(self, filename: str) -> str:
        return f"gs://{self.bucket_name}/{filename}"

    def download_file(
        self, filename: str, file: FileLikeObject
    ) -> Tuple[FileLikeObject, str]:
        try:
            uri = self._get_gs_uri(filename)
            logger.debug(f"download_file {uri}")
            file, _ = self.client.download_to_file(uri, file)
            return file, uri
        except Exception as e:
            logger.error(f"Failed to download file from {uri}: {e}")

    def upload_file(self, name: str, file: FileLikeObject) -> str:
        try:
            uri = self._get_gs_uri(name)
            logger.debug(f"upload_file {uri}")
            self.client.upload_file(uri, file, self.content_type)
            return uri
        except Exception as e:
            logger.error(f"Failed to upload file to {uri}: {e}")
