# test_integration_038_legal.py
import pytest

# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
from auth import auth
import common
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

TEST_DATA_DIR = Path(__file__).resolve().parent / "templates"

client = TestClient(app)


@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"


@pytest.mark.dependency(depends=["require_token"])
def test_legal_tags(token, data_partition):
    """
    Check legal tags
    Legal is not used directly by policy service, but it is used by OPA in dataauthz.
    So it's helpful to make sure the legal service is working as you'd expect or other integration tests will likely fail.
    """
    if conf.ENABLE_DEV_DIAGNOSTICS is False:
        pytest.skip("Skipping diagnostic test on legal tags - ENABLE_DEV_DIAGNOSTICS")

    if conf.LEGAL_BASE_URL is None:
        pytest.skip("NO LEGAL_BASE_URL")

    url = conf.LEGAL_BASE_URL + "/api/legal/v1/legaltags?valid=true"
    r = requests.get(
        url,
        headers={
            "Authorization": "Bearer " + token,
            "data-partition-id": data_partition,
        },
    )
    # print(json.dumps(r.json(), indent=4))

    count = 0
    for name in r.json()["legalTags"]:
        tags = name["name"]

        retval = common.get_legal_tag_info(
            data_partition_id=data_partition, access_token=token, legal_tags=[tags]
        )
        if retval:
            for retname in retval:
                x = retname["name"]
                assert x == tags, f"Expected to get {tags} from legal {retval}"
                count += 1
                if count > 10:
                    break
    assert count > 0, f"Expected some legal tags to exists. Found: {count}"


@pytest.mark.dependency(depends=["require_token"])
def test_partition(token, data_partition):
    """
    Check partition
    Partition service is not used by policy service, but it is used by search and is helpful for testing
    """

    pytest.skip("Skipping testing of partitions service")
    url = conf.PARTITION_API + "/partitions/" + data_partition
    print(f"URL: {url}")
    r = requests.get(
        url,
        headers={
            "Authorization": "Bearer " + token,
            "data-partition-id": data_partition,
        },
    )

    print(r.text)
    print(json.dumps(r.json(), indent=4))
