package osdu.instance.dataauthz

import data.osdu.instance.entitlements
import data.osdu.instance.legal

records[response] {
    id := input.records[_].id
    errors := [e | e := check_errors[_]
                   e.id == id]

    response := {
        "id" : id,
        "errors" : errors
    }
}

legalApiResponse := legal.groups
invalidLegaltagsSet := {l | l := legalApiResponse.body.invalidLegalTags[_].name}

entitlementsApiResponse := entitlements.groups
permissionSet := {e | e := entitlementsApiResponse.body.groups[_].email}

default root_user := false

root_user := true {
    startswith(permissionSet[_], "users.data.root@")
}

check_errors[{"id": id, "message": "User is not authorized to create or update records.", "reason": "User Unauthorized", "code": "401"}] {
    input.operation == ["view"][_]
    owners := {o | o := input.records[x].acls.owners[_]}
    viewers := { v | v := input.records[x].acls.viewers[_]}
    count(owners & permissionSet) == 0
    count(viewers & permissionSet) == 0
    root_user == false

    id := input.records[x].id
}

check_errors[{"id": id, "message": "The user is not authorized to perform this action", "reason": "Access denied", "code": "403"}] {
    input.operation == ["update", "purge", "delete"][_]
    owners := {o | o := input.records[x].acls.owners[_]}
    count(owners & permissionSet) == 0
    root_user == false

    id := input.records[x].id
}

check_errors[{"id": id, "message": "Invalid legal tags found on record", "reason": "Invalid legal tags", "code": "400"}] {
    input.operation == ["create"][_]
    recordsLegalTags := { r | r := input.records[x].legal.legaltags[_] }
    count(recordsLegalTags & invalidLegaltagsSet) > 0

    id := input.records[x].id
}

check_errors[{"id": id, "message": message, "reason": "Unauthorized", "code": code}] {
    entitlementsApiResponse.status_code != 200

    id := input.records[x].id
    code := entitlementsApiResponse.status_code
    message := sprintf("Entitlements response %v %v",[entitlementsApiResponse.status_code, entitlementsApiResponse.raw_body])
}

check_errors[{"id": id, "message": message, "reason": "Error from compliance service", "code": code}] {
    legalApiResponse.status_code != 200

    id := input.records[x].id
    code := legalApiResponse.status_code
    message := sprintf("Legal response %v %v",[legalApiResponse.status_code, legalApiResponse.raw_body])
}

check_errors[{"id": id, "message": "Invalid 'operation' supplied on input", "reason": "Bad Request", "code": "400"}] {
    input.operation != "create"
    input.operation != "update"
    input.operation != "view"
    input.operation != "purge"
    input.operation != "delete"

    id := input.records[x].id
}
